@@ -30,25 +30,37 @@ public:
   }
 };
 
+class pixelData {
+public:
+  GLfloat position[3];
+  GLint  material;
+  GLfloat width;
+  pixelData() {};
+  pixelData(const triple& v, double width) : width(width) {
+    position[0]=v.getx();
+    position[1]=v.gety();
+    position[2]=v.getz();
+    material=drawElement::materialIndex;
+  }
+};
+
 struct BezierCurve
 {
   static std::vector<vertexData1> vertexbuffer;
   static std::vector<GLuint> indices;
-  GLuint nvertices;
   double res,res2;
   triple Min,Max;
 
   static GLuint vertsBufferIndex; 
   static GLuint elemBufferIndex; 
   
-  BezierCurve() : nvertices(0) {}
-  
   void init(double res, const triple& Min, const triple& Max);
     
 // Store the vertex v in the buffer.
-  GLuint vertex(const triple &v) {
+  static GLuint vertex(const triple &v) {
+    size_t nvertices=vertexbuffer.size();
     vertexbuffer.push_back(vertexData1(v));
-    return nvertices++;
+    return nvertices;
   }
   
   void createBuffers() {
@@ -74,13 +86,9 @@ struct BezierCurve
       Z < Min.getz() || z > Max.getz();
   }
   
-  void clear() {
-    nvertices=0;
+  static void clear() {
     vertexbuffer.clear();
     indices.clear();
-    
-    glDeleteBuffers(1,&vertsBufferIndex);
-    glDeleteBuffers(1,&elemBufferIndex);
   }
   
   ~BezierCurve() {}
@@ -104,10 +112,22 @@ struct BezierCurve
 
 struct Pixel
 {
+  static std::vector<pixelData> vertexbuffer;
+  
+// Store the vertex v in the buffer.
+  static void vertex(const triple &v, double width) {
+    vertexbuffer.push_back(pixelData(v,width));
+  }
+  
+  static void clear() {
+    vertexbuffer.clear();
+  }
+  
   Pixel() {}
   ~Pixel() {}
   
-  void draw(const triple& p);
+  void queue(const triple& p, double width);
+  void draw();
 };
 
 #endif
