@@ -15,14 +15,6 @@ using ::orient3d;
 
 #ifdef HAVE_GL
 
-
-//FIXME: Find a nicer way to pass the shader program number,
-// preferably without global variables...
-
-extern GLint noColorShader;
-extern GLint colorShader;
-extern void setUniforms(GLint shader); 
-
 const size_t nbuffer=10000; // Initial size of dynamic buffers
 
 std::vector<vertexData> BezierPatch::vertexbuffer;
@@ -71,8 +63,6 @@ struct iz {
 
 std::vector<iz> IZ;
 
-extern const double Fuzz2;
-
 const double FillFactor=0.1;
 const double BezierFactor=0.4;
 
@@ -534,7 +524,7 @@ void BezierPatch::render(const triple *p, bool straight, GLfloat *c0)
   for(unsigned i=1; i < 16; ++i)
     epsilon=max(epsilon,abs2(p[i]-p0));
   
-  epsilon *= Fuzz2;
+  epsilon *= Fuzz4;
     
   triple p3=p[3];
   triple p12=p[12];
@@ -793,7 +783,7 @@ void BezierTriangle::render(const triple *p, bool straight, GLfloat *c0)
   for(int i=1; i < 10; ++i)
     epsilon=max(epsilon,abs2(p[i]-p0));
   
-  epsilon *= Fuzz2;
+  epsilon *= Fuzz4;
     
   GLuint I0,I1,I2;
     
@@ -895,7 +885,7 @@ void bounds(const std::vector<GLuint>& I)
   
 void BezierPatch::drawMaterials()
 {
-  if(nvertices == 0)
+  if(indices.size() == 0)
     return;
   
   static const size_t size=sizeof(GLfloat);
@@ -903,7 +893,7 @@ void BezierPatch::drawMaterials()
 
   GLuint vertsBufferIndex; 
   GLuint elemBufferIndex; 
-  
+
   GLuint vao;
   
   glGenVertexArrays(1,&vao);
@@ -915,15 +905,15 @@ void BezierPatch::drawMaterials()
   registerBuffer(vertexbuffer,vertsBufferIndex);
   registerBuffer(indices,elemBufferIndex);
   
-  camp::setUniforms(noColorShader); 
-
-  const GLint posAttrib=glGetAttribLocation(noColorShader,"position");
-  const GLint normalAttrib=glGetAttribLocation(noColorShader,"normal");
-  const GLint materialAttrib=glGetAttribLocation(noColorShader,"material");
-  
   glBindBuffer(GL_ARRAY_BUFFER,vertsBufferIndex);
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,elemBufferIndex);
 
+  camp::setUniforms(materialShader); 
+
+  const GLint posAttrib=glGetAttribLocation(materialShader,"position");
+  const GLint normalAttrib=glGetAttribLocation(materialShader,"normal");
+  const GLint materialAttrib=glGetAttribLocation(materialShader,"material");
+  
   glVertexAttribPointer(posAttrib,3,GL_FLOAT,GL_FALSE,bytestride,(void *) 0);
   glEnableVertexAttribArray(posAttrib);
     
@@ -942,17 +932,14 @@ void BezierPatch::drawMaterials()
   glDisableVertexAttribArray(normalAttrib);
   glDisableVertexAttribArray(materialAttrib);
 
+  deleteUniforms();
+  
   glBindBuffer(GL_ARRAY_BUFFER,0);
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);
-  glUseProgram(0);
 
   glBindVertexArray(0);
   glDeleteVertexArrays(1,&vao);
   
-  nvertices=0;
-  vertexbuffer.clear();
-  indices.clear();
-  
   glDeleteBuffers(1,&vertsBufferIndex);
   glDeleteBuffers(1,&elemBufferIndex);
 }
@@ -961,7 +948,7 @@ void BezierPatch::drawColors(GLuint& Nvertices,
                              std::vector<VertexData>& Vertexbuffer,
                              std::vector<GLuint>& Indices)
 {
-  if(Nvertices == 0)
+  if(Indices.size() == 0)
     return;
 
   static const size_t size=sizeof(GLfloat);
@@ -981,6 +968,9 @@ void BezierPatch::drawColors(GLuint& Nvertices,
   registerBuffer(Vertexbuffer,vertsBufferIndex);
   registerBuffer(Indices,elemBufferIndex);
   
+  glBindBuffer(GL_ARRAY_BUFFER,vertsBufferIndex);
+  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,elemBufferIndex);
+
   camp::setUniforms(colorShader); 
 
   const GLint posAttrib=glGetAttribLocation(colorShader,"position");
@@ -988,9 +978,6 @@ void BezierPatch::drawColors(GLuint& Nvertices,
   const GLint colorAttrib=glGetAttribLocation(colorShader,"color");
   const GLint materialAttrib=glGetAttribLocation(colorShader,"material");
   
-  glBindBuffer(GL_ARRAY_BUFFER,vertsBufferIndex);
-  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,elemBufferIndex);
-
   glVertexAttribPointer(posAttrib,3,GL_FLOAT,GL_FALSE,bytestride,
                         (void *) 0);
   glEnableVertexAttribArray(posAttrib);
@@ -1015,17 +1002,14 @@ void BezierPatch::drawColors(GLuint& Nvertices,
   glDisableVertexAttribArray(colorAttrib);
   glDisableVertexAttribArray(materialAttrib);
 
+  deleteUniforms();
+
   glBindBuffer(GL_ARRAY_BUFFER,0);
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);
-  glUseProgram(0);
 
   glBindVertexArray(0);
   glDeleteVertexArrays(1,&vao);
   
-  Nvertices=0;
-  Vertexbuffer.clear();
-  Indices.clear();
-  
   glDeleteBuffers(1,&vertsBufferIndex);
   glDeleteBuffers(1,&elemBufferIndex);
 }
