@@ -20,8 +20,9 @@ GLuint compileAndLinkShader(std::vector<ShaderfileModePair> const& shaders,
   GLuint mainShader = glCreateProgram();
   std::vector<GLuint> compiledShaders;
 
-  for (auto const& shaderInfo : shaders) {
-    GLint newshader=createShaderFile(shaderInfo.first,shaderInfo.second,
+  size_t n=shaders.size();
+  for(size_t i=0; i < n; ++i) {
+    GLint newshader=createShaderFile(shaders[i].first,shaders[i].second,
                                      NLights,NMaterials,defineflags);
     glAttachShader(mainShader,newshader);
     compiledShaders.push_back(newshader);
@@ -29,9 +30,9 @@ GLuint compileAndLinkShader(std::vector<ShaderfileModePair> const& shaders,
 
   glLinkProgram(mainShader);
 
-  for (auto const& compiledShader : compiledShaders) {
-    glDetachShader(mainShader,compiledShader);
-    glDeleteShader(compiledShader);
+  for(size_t i=0; i < n; ++i) {
+    glDetachShader(mainShader,compiledShaders[i]);
+    glDeleteShader(compiledShaders[i]);
   }
 
   return mainShader;
@@ -88,8 +89,9 @@ GLuint createShaderFile(std::string file, int shaderType, size_t Nlights,
   shaderSrc << "#extension GL_ARB_shading_language_packing : enable"
             << "\r\n";
 
-  for (std::string const& flag : defineflags) {
-    shaderSrc << "#define " << flag << "\r\n";
+  size_t n=defineflags.size();
+  for(size_t i=0; i < n; ++i) {
+    shaderSrc << "#define " << defineflags[i] << "\r\n";
   }
 
   shaderSrc << "const int Nlights=" << Nlights << ";\r\n";
